<?php

namespace EventPortal\Http\Controllers\Admin;

use EventPortal\Domain\Component\Button;
use EventPortal\Domain\Delegate\Events\DelegateCreated;
use EventPortal\Domain\Delegate\Events\DelegateEdited;
use EventPortal\Domain\Delegate\Models\Delegate;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Form\Facades\Form;
use EventPortal\Domain\Form\Facades\FormSubmission;
use EventPortal\Domain\Table\Facades\Table;
use EventPortal\Domain\Util\Breadcrumb;
use EventPortal\Domain\Util\DefaultFields;
use EventPortal\Enums\InertiaMethod;
use Illuminate\Http\RedirectResponse;
use Inertia\Inertia;
use Inertia\Response;

class EventDelegateController
{
    public function index(Event $event): Response
    {
        $table = Table::make(Delegate::class)
            ->columns(request('cols'))
            ->size(request('size'))
            ->page(request('page'))
            ->search(request('query'))
            ->filters(request('filter'))
            ->get();

        $breadcrumbs = Breadcrumb::make()
            ->add('Event', route('admin.event.index', $event))
            ->add('Delegates')
            ->get();

        return Inertia::render('EventDelegate/Index', [
            'data' => $table,
            'breadcrumbs' => $breadcrumbs,
            'event' => $event,
        ]);
    }

    public function create(Event $event): Response
    {
        $fields = DefaultFields::forCreation(Delegate::class);

        $form = Form::fromDefaultFields($fields, Delegate::class)
            ->addAction(static function () use ($event) {
                return Button::make('Cancel')
                    ->variant('secondary')
                    ->method(InertiaMethod::VISIT())
                    ->route('admin.event.delegate.index', ['event' => $event]);
            })
            ->addAction(static function () use ($event) {
                return Button::make('Create Delegate')
                    ->method(InertiaMethod::POST())
                    ->route('admin.event.delegate.store', ['event' => $event]);
            });

        $breadcrumbs = Breadcrumb::make()
            ->add('Delegates', route('admin.event.delegate.index', $event))
            ->add('Create a delegate')
            ->get();

        return Inertia::render('EventDelegate/Create', [
            'form' => $form,
            'breadcrumbs' => $breadcrumbs,
        ]);
    }

    public function store(Event $event): RedirectResponse
    {
        $fields = DefaultFields::forStoring(Delegate::class);

        $form = Form::fromDefaultFields($fields, Delegate::class);

        $delegate = FormSubmission::store($form, request()->input());

        $event->delegates()->save($delegate, [
            'status' => Delegate::STATUS_ADDED,
        ]);

        event(new DelegateCreated($delegate));

        return redirect()->route('admin.event.delegate.index', $event);
    }

    public function edit(Event $event, Delegate $delegate): Response
    {
        $form = Form::fromModel($delegate)
            ->addAction(static function () use ($event): Button {
                return Button::make('Cancel')
                    ->variant('secondary')
                    ->method(InertiaMethod::VISIT())
                    ->route('admin.event.delegate.index', ['event' => $event]);
            })
            ->addAction(static function () use ($delegate, $event): Button {
                return Button::make('Update Delegate')
                    ->method(InertiaMethod::POST())
                    ->route('admin.event.delegate.update', ['delegate' => $delegate, 'event' => $event]);
            });

        $breadcrumbs = Breadcrumb::make()
            ->add('Delegates', route('admin.event.delegate.index', $event))
            ->add('Edit delegate')
            ->get();

        return Inertia::render('EventDelegate/Edit', [
            'form' => $form,
            'breadcrumbs' => $breadcrumbs,
            'delegateName' => $form->getValue('first_name'),
        ]);
    }

    public function update(Event $event, Delegate $delegate): RedirectResponse
    {
        $form = Form::fromModel($delegate);

        FormSubmission::update($form, request()->input());

        event(new DelegateEdited($delegate));

        return redirect()->route('admin.event.delegate.index', $event);
    }

    public function show(Event $event, Delegate $delegate): Response
    {
        $form = Form::display($delegate)
            ->addAction(static function () use ($event, $delegate) {
                return Button::make('Delete')
                    ->variant('secondary')
                    ->method(InertiaMethod::POST())
                    ->route('admin.event.delegate.delete', ['event' => $event, 'delegate' => $delegate]);
            })
            ->addAction(static function () use ($event, $delegate) {
                return Button::make('Edit')
                    ->variant('secondary')
                    ->method(InertiaMethod::VISIT())
                    ->route('admin.event.delegate.edit', ['event' => $event->id, 'delegate' => $delegate]);
            });

        $breadcrumbs = Breadcrumb::make()
            ->add('Delegates', route('admin.event.delegate.index', $event))
            ->add($form->getValue('first_name') . $form->getValue('last_name'))
            ->get();

        return Inertia::render('Event/Show', [
            'data' => $form,
            'breadcrumbs' => $breadcrumbs,
            'eventName' => $form->getValue('name'),
        ]);
    }
}
