<?php

namespace EventPortal\Tests\Feature\Admin\EventDelegate;

use EventPortal\Domain\Delegate\Models\Delegate;
use EventPortal\Domain\Delegate\Notifications\NotifyOfEdit;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\DatabaseMigrations;
use Illuminate\Support\Facades\Notification;
use Spatie\Snapshots\MatchesSnapshots;

class EventDelegateEditTest extends TestCase
{
    use DatabaseMigrations, MatchesSnapshots;

    /** @test **/
    public function can_edit_delegate(): void
    {
        $this->withoutExceptionHandling();

        $event = factory(Event::class)->create();
        $delegate = factory(Delegate::class)->create();

        $event->delegates()->save($delegate, [
            'status' => Delegate::STATUS_ADDED,
        ]);

        $this->assertMatchesJsonSnapshot($this->get(route('admin.event.delegate.edit', ['delegate' => $delegate, 'event' => $event]))->props());
    }

    /** @test */
    public function delegate_receives_notification_on_update(): void
    {
        Notification::fake();

        $event = factory(Event::class)->create();
        $delegate = factory(Delegate::class)->state('default-data')->create();

        $event->delegates()->save($delegate, [
            'status' => Delegate::STATUS_ADDED,
        ]);

        $this->post(route('admin.event.delegate.update', ['event' => $event, 'delegate' => $delegate]), [
            'first_name' => 'Test',
            'last_name' => 'Delegate',
            'profession' => 'Tester',
        ]);

        $this->assertCount(1, Delegate::all());
        $this->assertCount(1, $event->fresh()->delegates);

        $delegate = Delegate::first();
        Notification::assertSentTo($delegate, NotifyOfEdit::class);

        $this->assertDatabaseHas('event_user', [
            'user_id' => $delegate->id,
            'event_id' => $event->id,
            'status' => Delegate::STATUS_ADDED,
        ]);
    }

    /** @test */
    public function delegate_is_updated(): void
    {
        $this->markTestSkipped('DEPENDS: MEPS-251. Need "magic" accessor to validate presence of updates');
        Notification::fake();

        $event = factory(Event::class)->create();
        $delegate = factory(Delegate::class)->state('default-data')->create();
        $this->assertCount(0, $event->delegates);

        $event->delegates()->save($delegate, [
            'status' => Delegate::STATUS_ADDED,
        ]);

        $this->assertNull($delegate->first_name);
        $this->assertNull($delegate->last_name);
        $this->assertNull($delegate->profession);

        $this->post(route('admin.event.delegate.update', ['event' => $event, 'delegate' => $delegate]), [
            'first_name' => 'Test',
            'last_name' => 'Delegate',
            'profession' => 'Tester',
        ]);

        $this->assertEquals('Test', $delegate->first_name);
        $this->assertEquals('Delegate', $delegate->last_name);
        $this->assertEquals('Tester', $delegate->profession);
    }
}
