<?php

namespace EventPortal\Tests\Unit\Util;

use EventPortal\Domain\Component\Option;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Field\Enums\FieldType;
use EventPortal\Exceptions\NoDefaultConfig;
use EventPortal\Domain\Util\DefaultFields;
use EventPortal\Tests\TestCase;

class DefaultFieldsTest extends TestCase
{
    /** @test **/
    public function exception_thrown_when_no_config()
    {
        $this->expectException(NoDefaultConfig::class);

        config()->set('defaults.event.fields', null);

        DefaultFields::forCreation(Event::class)->get();
    }

    /** @test **/
    public function does_not_return_field_that_is_hidden()
    {
        $data = [
            [
                'label' => 'Key details',
                'component' => FieldType::Section()->getValue(),
                'children' => [
                    [
                        'label' => 'Event name',
                        'name' => 'name',
                        'component' => FieldType::TextInput()->getValue(),
                        'value' => 'Hello World!'
                    ],
                    [
                        'label' => 'Event status',
                        'name' => 'status',
                        'component' => FieldType::Select()->getValue(),
                        'options' => [
                            Option::make('Draft'),
                            Option::make('Published'),
                        ],
                        'hideWhen' => [
                            'create',
                        ],
                    ],
                ],
            ]
        ];

        config()->set('defaults.event.fields', $data);

        $actual = DefaultFields::forCreation(Event::class)->get();

        $expected = [
            [
                'label' => 'Key details',
                'component' => FieldType::Section()->getValue(),
                'children' => [
                    [
                        'label' => 'Event name',
                        'name' => 'name',
                        'component' => FieldType::TextInput()->getValue(),
                        'value' => 'Hello World!'
                    ],
                ],
            ]
        ];

        $this->assertEquals($expected, $actual);
    }

    /** @test **/
    public function does_not_return_section_that_is_hidden()
    {
        $data = [
            [
                'label' => 'Key details',
                'component' => FieldType::Section()->getValue(),
                'children' => [
                    [
                        'label' => 'Event name',
                        'name' => 'name',
                        'component' => FieldType::TextInput()->getValue(),
                    ],
                ],
            ],
            [
                'label' => 'Other',
                'component' => FieldType::Section()->getValue(),
                'hideWhen' => [
                    'create',
                ],
                'children' => [
                    [
                        'label' => 'Hidden Field',
                        'name' => 'hidden',
                        'component' => FieldType::TextInput()->getValue(),
                    ],
                ],
            ]
        ];

        config()->set('defaults.event.fields', $data);

        $actual = DefaultFields::forCreation(Event::class)->get();

        $expected = [
            [
                'label' => 'Key details',
                'component' => FieldType::Section()->getValue(),
                'children' => [
                    [
                        'label' => 'Event name',
                        'name' => 'name',
                        'component' => FieldType::TextInput()->getValue(),
                    ],
                ],
            ]
        ];

        $this->assertEquals($expected, $actual);
    }

    /** @test **/
    public function hideWhen_can_be_a_string()
    {
        $data = [
            [
                'label' => 'Key details',
                'component' => FieldType::Section()->getValue(),
                'children' => [
                    [
                        'label' => 'Event name',
                        'name' => 'name',
                        'component' => FieldType::TextInput()->getValue(),
                        'value' => 'Hello World!'
                    ],
                    [
                        'label' => 'Event status',
                        'name' => 'status',
                        'component' => FieldType::Select()->getValue(),
                        'options' => [
                            Option::make('Draft'),
                            Option::make('Published'),
                        ],
                        'hideWhen' => 'create',
                    ],
                ],
            ]
        ];

        config()->set('defaults.event.fields', $data);

        $actual = DefaultFields::forCreation(Event::class)->get();

        $expected = [
            [
                'label' => 'Key details',
                'component' => FieldType::Section()->getValue(),
                'children' => [
                    [
                        'label' => 'Event name',
                        'name' => 'name',
                        'component' => FieldType::TextInput()->getValue(),
                        'value' => 'Hello World!'
                    ],
                ],
            ]
        ];

        $this->assertEquals($expected, $actual);
    }
}

